@   echo off

:main /? | [/v | /unpack]
:: = DESCRIPTION
:: =   !PROG_NAME! - a script to install MinGW (32-bit).
:: =
:: =   In short, the script does the following:
:: =
:: =     .  Unpacks the Zip-archive containing the MinGW install utility into the
:: =        MinGW home directory.
:: =     .  Creates the initial package catalogue
:: =     .  Updates existing packages.
:: =     .  Installs the GNU C and C++ compiler suites.
:: =     .  Installs the basic MSYS package.
:: =     .  Appends !MINGW_HOME!\bin and !MSYS_HOME!\bin to the user's system-wide
:: =        PATH environment variable.
:: =          
:: = OPTIONS
:: =   /v       Be verbose.
:: =   /unpack  Only unpack the files into !MINGW_HOME!.
:: =
:: = FILES
:: =   mingw-get-x.x.x-mingw32-*.zip
:: =     (available at !MINGW32_GET_URL!)
:: =   !def_cat!
:: =   !sys_cat!
:: =
:: = VERSION
:: =   @(#) Version: 2015-11-19
:: =
:: = AUTHOR
:: =   Written by MiniMax.

    verify other 2> NUL:
    setlocal EnableExtensions
    if ErrorLevel 1 (
	echo Error - Unable to enable extensions.
	goto :EOF
    )

:defaults
    (set PROG_NAME=%~n0)
    (set PROG_DIR=%~dp0)

    if /i "%~1" == "/trace" shift & prompt $G$G & echo on

    PATH %PROG_DIR%\cmd-lib.lib;%PATH%

    if not defined MINGW_HOME	(set MINGW_HOME=%SystemDrive%\MinGW)
    if not defined MSYS_HOME	(set MSYS_HOME=%MINGW_HOME%\MSYS\1.0)
    if not defined UNPACKER	(set UNPACKER=%ProgramFiles%\7-Zip\7z)
    if not defined INSTALL_OPTS	(set INSTALL_OPTS=--desktop=all-users --start-menu=all-users)

    (set MINGW32_GET_URL=http://sourceforge.net/projects/mingw/files/Installer/mingw-get/mingw-get-0.6.2-beta-20131004-1/mingw-get-0.6.2-mingw32-beta-20131004-1-bin.zip)
rem (set MINGW32_SETUP_URL=http://downloads.sourceforge.net/project/mingw/Installer/mingw-get-setup.exe)
rem (set MINGW64_INSTALLER_URL=http://downloads.sourceforge.net/project/mingw-w64/Toolchains%20targetting%20Win32/Personal%20Builds/mingw-builds/installer/mingw-w64-install.exe)

    (set show_help=false)
    (set verbosity=0)
    (set unpack_only=false)

    (set def_cat=%MINGW_HOME%\var\lib\mingw-get\data\defaults.xml)
    (set sys_cat=%MINGW_HOME%\var\lib\mingw-get\data\profile.xml)

:getopts
    if /i "%~1" == "/?"		shift & (set show_help=true)		& goto :getopts

    if /i "%~1" == "/v"		shift & (set /a verbosity=verbosity+1)	& goto :getopts
    if /i "%~1" == "/unpack"	shift & (set unpack_only=true)		& goto :getopts

    set char1=%~1
    set char1=%char1:~0,1%

    if "%char1%" == "/" (
	echo Unknown option - %1.
	echo.
	call usage "%PROG_NAME%.cmd"
	call error_exit
	goto :EOF
    )

    if "%show_help%" == "true" call help "%PROG_NAME%.cmd" & goto :EOF

    call basename "%MINGW32_GET_URL%" >NUL:
    if not exist "%_%" (
	echo.
	echo ERROR: Unable to unpack file "%_%"
	echo because the file does not exist in "%CD%".
	echo.
	echo Please download it from "%MINGW32_GET_URL%"
	echo and then re-run this script.
	call error_exit
	goto :EOF
    )

    call :unpack "%_%" "%MINGW_HOME%" || goto :EOF

    if "%unpack_only%" == "true" goto :EOF

    if not exist %sys_cat% (
	if %verbosity% gtr 0 echo >&2 Copying default catalogue to %sys_cat%
	copy "%def_cat%" "%sys_cat%"
    )

    "%MINGW_HOME%"\bin\mingw-get %INSTALL_OPTS% update
    "%MINGW_HOME%"\bin\mingw-get %INSTALL_OPTS% install gcc g++
    "%MINGW_HOME%"\bin\mingw-get %INSTALL_OPTS% install msys-base

    call :xPATH "gcc.exe" "%MINGW_HOME%\bin"
    call :xPATH "sh.exe"  "%MSYS_HOME%\bin"
goto :EOF

:unpack file dst-dir
    echo Unpacking "%~1" into "%~2".

    if %verbosity% equ 0 (
	"%UNPACKER%" x                -y -o"%~2" "%~1" >NUL:
    ) else (
	"%UNPACKER%" x -bb%verbosity% -y -o"%~2" "%~1"
    )

    if errorlevel 1 (
	echo.
	echo ERROR: Unable to unpack file "%~1" into "%~2%"
	echo using the command:
	echo.
	echo.  %UNPACKER% x -o"%~2" "%~1"
	echo.
	echo Please check that you have defined UNPACKER correctly in %PROG_NAME%.cmd.
	echo Alternatively, you can manually unpack "%~1" into a folder called
	echo "%~2" and then re-run this script.
	call error_exit
	goto :EOF
    )
goto :EOF

:xPATH
    call :getx_var PATH
    set oPATH=%_%

    setlocal enabledelayedexpansion
    for %%I in ("%~1") do (
	if "" == "%%~$oPATH:I" (
	    call :getx_var PATH
	    echo Appending "%~2" to PATH.
	    echo Please exit this CMD for the changes to take effect.
	    if exist "%~2\%%I" setx PATH !_!;%~2
	)
    )
    endlocal
goto :EOF

:getx_var var-name
    set _=
    for /F "skip=2 tokens=3*" %%V in ('reg query "HKCU\Environment" /v "%~1"') do set _=%%V%%W
goto :EOF

rem .--------------------------------------------------------------------------
rem | Displays a selection of variables belonging to this script.
rem | Very handy when debugging.
rem '--------------------------------------------------------------------------
:dump_variables
    echo =======
    echo cwd            = "%CD%"
    echo tmp_dir        = "%tmp_dir%"

    echo PROG_DIR       = "%PROG_DIR%"
    echo PROG_NAME      = "%PROG_NAME%"

    echo MINGW_HOME     = "%MINGW_HOME%"
    echo MSYS_HOME      = "%MSYS_HOME%"
    echo UNPACKER       = "%UNPACKER%"
    echo INSTALL_OPTS   = "%INSTALL_OPTS%"

    echo verbosity      = "%verbosity%"
    echo unpack_only    = "%unpack_only%"

    echo MINGW32_GET_URL= "%MINGW32_GET_URL%"

    if defined tmp_dir if exist %tmp_dir%\NUL: (
	echo.
	dir %tmp_dir%
    )

    echo =======
goto :EOF

rem vim: set filetype=dosbatch tabstop=8 softtabstop=4 shiftwidth=4 noexpandtab:
rem vim: set foldmethod=indent
