@   echo off

:main /? | [/v] [/unpack]
:: = DESCRIPTION
:: =   !PROG_NAME! - a script to install SDL using MinGW with MSYS.
:: =
:: =   In short, the script does the following:
:: =
:: =     .  Unpacks the Zip-archive containing the SDL source code into the
:: =        MinGW 'src' directory.
:: =     .  Switches to the 'src' directory.
:: =     .  Prepares for compilation of SDL (./confgure).
:: =     .  Compiles SDL (make).
:: =     .  Installs SDL (make install).
:: =
:: =    For more information, read the INSTALL file in !MINGW_HOME!\src\SDL-x.x.x.
:: =          
:: = OPTIONS
:: =   /v       Be verbose.
:: =   /unpack  Only unpack the files into !MINGW_HOME!\src.
:: =
:: = FILES
:: =   SDL-x.x.x.zip
:: =     (available at !SDL_SRC_URL!)
:: =
:: = VERSION
:: =   @(#) Version: 2015-11-19
:: =
:: = AUTHOR
:: =   Written by MiniMax.

    verify other 2> NUL:
    setlocal EnableExtensions
    if ErrorLevel 1 (
	echo Error - Unable to enable extensions.
	goto :EOF
    )

:defaults
    (set PROG_NAME=%~n0)
    (set PROG_DIR=%~dp0)

    if /i "%~1" == "/trace" shift & prompt $G$G & echo on

    PATH %PROG_DIR%\cmd-lib.lib;%PATH%

    if not defined MINGW_HOME	(set MINGW_HOME=%SystemDrive%\MinGW)
    if not defined MSYS_HOME	(set MSYS_HOME=%MINGW_HOME%\MSYS\1.0)
    if not defined UNPACKER	(set UNPACKER=%ProgramFiles%\7-Zip\7z)
    if not defined INSTALL_OPTS	(set INSTALL_OPTS=--desktop=all-users --start-menu=all-users)

    (set SDL_SRC_URL=http://www.libsdl.org/release/SDL-1.2.15.zip)

    (set show_help=false)
    (set verbosity=0)
    (set unpack_only=false)

:getopts
    if /i "%~1" == "/?"		shift & (set show_help=true)		& goto :getopts

    if /i "%~1" == "/v"		shift & (set /a verbosity=verbosity+1)	& goto :getopts
    if /i "%~1" == "/unpack"	shift & (set unpack_only=true)		& goto :getopts

    set char1=%~1
    set char1=%char1:~0,1%

    if "%char1%" == "/" (
	echo Unknown option - %1.
	echo.
	call usage "%PROG_NAME%.cmd"
	call error_exit
	goto :EOF
    )

    if "%show_help%" == "true" call help "%PROG_NAME%.cmd" & goto :EOF

    call basename "%SDL_SRC_URL%" >NUL:
    if not exist "%_%" (
	echo.
	echo ERROR: Unable to unpack file "%_%"
	echo because the file does not exist in "%CD%".
	echo.
	echo Please download it from "%SDL_SRC_URL%"
	echo and then re-run this script.
	call error_exit
	goto :EOF
    )

    if not exist %MINGW_HOME%\NUL: (
	echo ERROR: "%MINGW_HOME%" does not exist ^(or is not a directory^).
	call error_exit
	goto :EOF
    )

    call :unpack "%_%" "%MINGW_HOME%\src" "%SDL_SRC_URL%" || goto :EOF

    if "%unpack_only%" == "true" goto :EOF

    call basename "%SDL_SRC_URL%" .zip >NUL:
    cd "%MINGW_HOME%\src\%_%"

    sh -c ./configure
    sh -c make
    sh -c make install
goto :EOF

:unpack file dst-dir
    echo Unpacking "%~1" into "%~2".

    if %verbosity% equ 0 (
	"%UNPACKER%" x                -y -o"%~2" "%~1" >NUL:
    ) else (
	"%UNPACKER%" x -bb%verbosity% -y -o"%~2" "%~1"
    )

    if errorlevel 1 (
	echo.
	echo ERROR: Unable to unpack file "%~1" into "%~2%"
	echo using the command:
	echo.
	echo.  %UNPACKER% x -o"%~2" "%~1"
	echo.
	echo Please check that you have defined UNPACKER correctly in %PROG_NAME%.cmd.
	echo Alternatively, you can manually unpack "%~1" into a folder called
	echo "%~2" and then re-run this script.
	call error_exit
	goto :EOF
    )
goto :EOF

rem .--------------------------------------------------------------------------
rem | Displays a selection of variables belonging to this script.
rem | Very handy when debugging.
rem '--------------------------------------------------------------------------
:dump_variables
    echo =======
    echo cwd            = "%CD%"
    echo tmp_dir        = "%tmp_dir%"

    echo PROG_DIR       = "%PROG_DIR%"
    echo PROG_NAME      = "%PROG_NAME%"

    echo MINGW_HOME     = "%MINGW_HOME%"
    echo MSYS_HOME      = "%MSYS_HOME%"
    echo UNPACKER       = "%UNPACKER%"
    echo INSTALL_OPTS   = "%INSTALL_OPTS%"

    echo verbosity      = "%verbosity%"
    echo unpack_only    = "%unpack_only%"

    echo SDL_SRC_URL    = "%SDL_SRC_URL%"

    if defined tmp_dir if exist %tmp_dir%\NUL: (
	echo.
	dir %tmp_dir%
    )

    echo =======
goto :EOF

rem vim: set filetype=dosbatch tabstop=8 softtabstop=4 shiftwidth=4 noexpandtab:
rem vim: set foldmethod=indent
